import time
import gc
import rtc
import board
import busio
from adafruit_display_text import label
from adafruit_display_shapes.roundrect import RoundRect
import terminalio
import displayio
import keypad
from adafruit_st7789 import ST7789
from adafruit_bme280 import basic as adafruit_bme280

#I2C Pin of the PicoBoy - Color
SDA = board.GP20
SCL = board.GP21
# Create sensor object, using the board's default I2C bus.
i2c = busio.I2C(SCL, SDA) 
bme280 = adafruit_bme280.Adafruit_BME280_I2C(i2c, address=0x77)

# change this to match the location's pressure (hPa) at sea level
bme280.sea_level_pressure = 1030

# 1,69 Zoll 240x280 Pixel ST7789 Display
# SPI-Bus,  in Landscape format
dc=board.GP8
reset=board.GP9
cs=board.GP10
sck=board.GP18
mosi=board.GP19
bl=board.GP26
# Release any resources currently in use for the displays
displayio.release_displays()
spi = busio.SPI(sck, mosi)
display_bus = displayio.FourWire(spi, command=dc, chip_select=cs, reset=reset)
display = ST7789(display_bus, rotation=90, width=280, height=240, backlight_pin=bl, rowstart=20, colstart=0)
display.brightness = 1
# Make the display context
splash = displayio.Group()
timeset= displayio.Group()
display.root_group = splash

K_UP = 0x04
K_DOWN = 0x02
K_RIGHT = 0x08
K_LEFT = 0x01
K_A = 0x10
K_B = 0x20
K_Z = 0x40

class _Buttons:
    def __init__(self):
        self.keys = keypad.Keys((board.GP4, board.GP3, board.GP1, board.GP2, board.GP27, board.GP28, board.GP0),
                                value_when_pressed=False, interval=0.05)
        self.last_state = 0
        self.event = keypad.Event(0, False)
        self.last_z_press = None

    def get_pressed(self):
        buttons = self.last_state
        events = self.keys.events
        while events:
            if events.get_into(self.event):
                bit = 1 << self.event.key_number
                if self.event.pressed:
                    buttons |= bit
                    self.last_state |= bit
                else:
                    self.last_state &= ~bit
        return buttons
buttons = _Buttons()

# Make a background color fill
color_bitmap = displayio.Bitmap(display.width, display.height, 3)
color_palette = displayio.Palette(3)
color_palette[0] = 0x660088
color_palette[1] = 0x000000
color_palette[2] = 0xffffff
bg_sprite = displayio.TileGrid(color_bitmap, pixel_shader=color_palette, x=0, y=0)
splash.append(bg_sprite)

# Draw a label
text_area = label.Label(font=terminalio.FONT, text="\n   PicoBoy - Color\n\n", color=0xFFFF00, scale=2, line_spacing=1 )
text_group = displayio.Group(x=25, y=40)
text_group.append(text_area) 
splash.append(text_group)

# create the label for time in timeset
updating_label_instruction = label.Label(font=terminalio.FONT, text="", scale=2, color=0xffffcc, line_spacing=1)
updating_label_instruction.anchor_point = (0, 0)
updating_label_instruction.anchored_position = (30, 20)
timeset.append(updating_label_instruction)
# Umrandung
roundrect1 = RoundRect(25, 40, 230, 70, 10, fill=0x0, outline=0xffffff, stroke=3)
timeset.append(roundrect1)
# create the label Zeitstellen
updating_label_zeit = label.Label(font=terminalio.FONT, text="00:00:00", scale=4, color=0xffffff, line_spacing=1)
updating_label_zeit.anchor_point = (0, 0)
updating_label_zeit.anchored_position = (45, 50)
timeset.append(updating_label_zeit)
#create the label function
updating_label = label.Label(font=terminalio.FONT, scale=2, color=0x00ff00, line_spacing=1)
updating_label.text = "up   : hour up\ndown : hour down\nright: min up\nleft : min down\nexit : ok"
updating_label.anchor_point = (0, 0)
updating_label.anchored_position = (40, 120)
timeset.append(updating_label)

weekday=["Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Samstag","Sonntag"]

current_time = time.localtime()
if current_time.tm_year == 2020:
    with open("last_time.txt", "r") as f:
        hour = int(f.readline())
        minute = int(f.readline())
        mday = int(f.readline())
        mon = int(f.readline())
        wday = int(f.readline())
        year = int(f.readline())
    f.close()
    rc = rtc.RTC()
    rc.datetime = time.struct_time((year, mon, mday, hour, minute, 0, wday, 1, -1))
    second = current_time.tm_sec
else:
    hour = current_time.tm_hour
    minute = current_time.tm_min
    second = current_time.tm_sec
    mday = current_time.tm_mday
    mon = current_time.tm_mon
    wday = current_time.tm_wday
    year = current_time.tm_year

debounce = False
stellen = False

while True:
    # Zeit stellen
    cur_btn_vals = buttons.get_pressed()
    cur_up = cur_btn_vals & K_UP
    cur_left = cur_btn_vals & K_LEFT
    cur_right = cur_btn_vals & K_RIGHT
    cur_down = cur_btn_vals &  K_DOWN
    cur_a = cur_btn_vals & K_A
    cur_b = cur_btn_vals & K_B
    cur_exit = cur_btn_vals & K_Z
    # Stellen aufrufen
    if (cur_a or cur_b) and not debounce:
        stellen = True
        debounce = True
        display.root_group = timeset
    # Stellen verlassen
    if cur_exit and not debounce:
        stellen = False
        debounce = True
        current_time = time.localtime()
        with open("/last_time.txt", "w") as f:
            f.write(str(current_time.tm_hour)+"\n")
            f.write(str(current_time.tm_min)+"\n")
            f.write(str(current_time.tm_mday)+"\n")
            f.write(str(current_time.tm_mon)+"\n")
            f.write(str(current_time.tm_wday)+"\n")
            f.write(str(current_time.tm_year)+"\n")
        f.close()
        display.root_group = splash
    if stellen == True:
        updating_label_zeit.text = "{:02}:{:02}:00".format(current_time.tm_hour,current_time.tm_min)
        # Stunden erhoehen
        if cur_up and not debounce:
            debounce = True
            if hour < 23:
                hour += 1
            else:
                hour = 0
            r = rtc.RTC()
            r.datetime = time.struct_time((year, mon, mday, hour, minute, 0, wday, 1, -1))
        # Stunden verringern
        if cur_down and not debounce:
            debounce = True
            if hour > 0:
                hour -= 1
            else:
                hour = 23
            r = rtc.RTC()
            r.datetime = time.struct_time((year, mon, mday, hour, minute, 0, wday, 1, -1))
        # Minuten erhoehen
        if cur_right and not debounce:
            debounce = True
            if minute < 59:
                minute += 1
            else:
                minute = 0
            r = rtc.RTC()
            r.datetime = time.struct_time((year, mon, mday, hour, minute, 0, wday, 1, -1))
        # Minuten verringern
        if cur_left and not debounce:
            debounce = True
            if minute > 0:
                minute -= 1
            else:
                minute = 59
            r = rtc.RTC()
            r.datetime = time.struct_time((year, mon, mday, hour, minute, 0, wday, 1, -1))
    if cur_btn_vals == 0:
        debounce = False  
        
    if second == 59:       
        current_time = time.localtime()
        with open("/last_time.txt", "w") as f:
            f.write(str(current_time.tm_hour)+"\n")
            f.write(str(current_time.tm_min+1)+"\n")
            f.write(str(current_time.tm_mday)+"\n")
            f.write(str(current_time.tm_mon)+"\n")
            f.write(str(current_time.tm_wday)+"\n")
            f.write(str(current_time.tm_year)+"\n")
        f.close()
    
    current_time = time.localtime()
    second = current_time.tm_sec
    if second % 5:
        text_area.text = (weekday[current_time.tm_wday]
                        + " {:02}.{:02}.{:04}".format(current_time.tm_mday,current_time.tm_mon,current_time.tm_year)
                        + "\n{:02}:{:02} Uhr".format(current_time.tm_hour,current_time.tm_min)
                        + "\n\nTemper.: %0.1f C" % bme280.temperature
                        + "\n\nLuftdr.: %0.1f hPa" % bme280.pressure
                        + "\nHoehe  : %0.2f m" % bme280.altitude)
    gc.collect()   
    #print(gc.mem_free())